% LSE_LBF implements the level set evolution (LSE) for the method in Chunming Li et al's paper:
%       "Minimization of Region-Scalable Fitting Energy for Image Segmentation", 
%        IEEE Trans. Image Processing, vol. 17 (10), pp.1940-1949, 2008.
%
% Author: Chunming Li, all rights reserved
% E-mail: li_chunming@hotmail.com
% URL:  http://www.engr.uconn.edu/~cmli/
%
% Modified by Wendeson Oliveira. 

function u = RSF(u0,Img,KONE,KI,nu,timestep,mu,lambda1,lambda2,epsilon)
u=u0;

for k1=1:1
    u=NeumannBoundCond(u);
    K=curvature_central(u);
    DrcU=(epsilon/pi)./(epsilon^2.+u.^2);         
    
    [f1, f2] = localBinaryFit(Img, u, KI, KONE, epsilon, 3, 9);
   
    s1=lambda1.*f1.^2-lambda2.*f2.^2;                 
    s2=lambda1.*f1-lambda2.*f2;
  
    cv1 = MatchFilterWithGaussDerivativeORSF(s1,1,3,9);
    cv2 = MatchFilterWithGaussDerivativeORSF(s2,1,3,9);
    
    dataForce=(lambda1-lambda2)*KONE.*Img.*Img + cv1 - 2.*Img.*cv2;
     
    A=-DrcU.*dataForce;                                
    P=mu*(4*del2(u)-K);                                
    L=nu.*DrcU.*K;                                     
    u=u+timestep*(L+P+A);                              
    
end

function [f1, f2]= localBinaryFit(Img, u, KI, KONE, epsilon, x1, x2)
Hu=0.5*(1+(2/pi)*atan(u./epsilon));                    
I=Img.*Hu;

c1 = MatchFilterWithGaussDerivativeORSF(Hu,1,x1,x2);
c2 = MatchFilterWithGaussDerivativeORSF(I,1,x1,x2);

f1=c2./(c1);                                         
f2=(KI-c2)./(KONE-c1);                                 
                                                       
                                                         

function g = NeumannBoundCond(f)
[nrow,ncol] = size(f);
g = f;
g([1 nrow],[1 ncol]) = g([3 nrow-2],[3 ncol-2]);  
g([1 nrow],2:end-1) = g([3 nrow-2],2:end-1);          
g(2:end-1,[1 ncol]) = g(2:end-1,[3 ncol-2]);  

function k = curvature_central(u)                       
[ux,uy] = gradient(u);                                  
normDu = sqrt(ux.^2+uy.^2+1e-10);                       
                                                        
Nx = ux./normDu;                                       
Ny = uy./normDu;
[nxx,junk] = gradient(Nx);                              
[junk,nyy] = gradient(Ny);                              
k = nxx+nyy;

