% 
%INPUTS:
%   img         -   retinal image
%   mask        -   FOV mask image
%   threshold   -   threshold value;
%OUTPUT:
%   BW- segmented retinal image


function BW = medianRanking(img, mask, threshold)

    greenChannel = img(:,:,2);
    greenChannel = double(greenChannel);
    %CS = ContrastStretchNorm(greenChannel,0.12);
    CS = contrastStretching(greenChannel, [0 1]);

    se = strel('disk',5);
    mask = imerode(mask,se);
    
    CS = normalize(CS);

    MF = MatchFilterWithGaussDerivative(CS*255,1.5,5,8);
    MF = normalize(MF);

    FR = FrangiFilter2D(CS*255);
    FR = normalize(FR);

    GW = gaborWavelet(CS,5, 2, mask);
    GW = normalize(GW);

    CS = 1 - CS;

    [Y, r1] = sort(MF(mask==255), 'descend');
    [Y, r2] = sort(GW(mask==255), 'descend');
    [Y, r3] = sort(FR(mask==255), 'descend');
    [Y, r4] = sort(CS(mask==255), 'descend');

    idx = 1:length(Y);

    R1 = zeros(1,length(Y));
    R2 = zeros(1,length(Y));
    R3 = zeros(1,length(Y));
    R4 = zeros(1,length(Y));

    R1(r1) = length(Y)+1-idx;
    R2(r2) = length(Y)+1-idx;
    R3(r3) = length(Y)+1-idx;
    R4(r4) = length(Y)+1-idx;

    R1 = R1';
    R2 = R2';
    R3 = R3';
    R4 = R4';

    rank = [R1/length(Y) R2/length(Y) R3/length(Y) R4/length(Y)];

    medianRanking = median(rank');
    medianRanking = medianRanking';

    [Sx Sy] = size(mask);

    BW = zeros(Sx,Sy);
    BW(mask==255) = medianRanking>threshold;
    BW(mask==0) = 0;

    BW = postProcessing(BW,2);

end
